import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:ffi';
import 'dart:io';

import 'package:appmate_flutter/helpers/BaseResponse.dart';
import 'package:appmate_flutter/helpers/CreateUserRelationResponse.dart';
import 'package:appmate_flutter/helpers/EntitlementResponse.dart';
import 'package:appmate_flutter/helpers/GenericError.dart';
import 'package:appmate_flutter/helpers/MakePurchaseResponse.dart';
import 'package:appmate_flutter/helpers/OfferwallResponse.dart';
import 'package:appmate_flutter/helpers/ProductsResponse.dart';
import 'package:appmate_flutter/helpers/PurchasesResponse.dart';
import 'package:appmate_flutter/helpers/UserEventType.dart';
import 'package:appmate_flutter/models/Offerwall.dart';
import 'package:appmate_flutter/models/Product.dart';
import 'package:appmate_flutter/models/PurchaseInfo.dart';
import 'package:appmate_flutter/models/UserRelation.dart';
import 'package:flutter/services.dart';

/// A class used for creating and controlling in-app purchases.
/// PurchaseClient is the main class of Appmate.
/// Appmate helps developers to manage in-application purchases easily.
/// Thanks to Appmate, it is much easier to create and consume IAP products,
/// manage purchases, user relations and vice versa.
/// For further information you can view the detailed
///
/// See also:
///
///  * For web documentation [appmate](https://appmate.tech/docs/sdk/android)
class PurchaseClient {
  static const MethodChannel _channel = const MethodChannel('appmate_flutter');

  ///#### Sets  Appmate apiKey
  ///
  /// Api key can be retrieved from Appmate Portal via navigating to application page.
  ///
  /// [apiKey] Unique identifier for using Appmate services.
  ///
  /// Callbacks [BaseResponse] and [GenericError] in case of failure.
  static Future<BaseResponse> setApiKey(String apiKey) async {
    List<Object?>? response =
        await _channel.invokeMethod('setApiKey', {"apiKey": apiKey});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### For iOS platform only:
  /// This method can be used to fetch non-synched products from appmate.
  /// While it is true, non-synched products will also be able to fetched as a response;
  /// otherwise, only the products successfully synched with Apple Store can be fetched.
  /// The default parameter is false.

  /// The [isActive] value is used to activate the sandbox. While it is true,
  /// non-synched products will also be fetched.
  ///
  /// Callbacks [BaseResponse] and [GenericError] in case of failure.
  static Future<BaseResponse> setSandboxActive(bool isActive) async {
    List<Object?>? response =
        await _channel.invokeMethod('setSandboxActive', {"isActive": isActive});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// ####  Modifies  device user id. The default user id is set by device automatically.
  ///
  /// It's an optional parameter that uniquely identifies your users within your application.
  /// You should set it only if you have a membership system for your app.
  /// You don't have to add it to PurchaseClient's constructor when creating an instance.
  /// Alternatively, you could specify your own user identifiers with
  /// PurchaseClient.instance.setUserId() later in your app lifecycle.
  /// This method requires a userId as a type of String and should be a unique value.
  /// It's necessary to fetch products and make purchase.
  ///
  /// Device [userId] information.
  ///
  /// Callbacks [BaseResponse] and [GenericError] in case of failure.
  static Future<BaseResponse> setUserId(String userId) async {
    List<Object?>? response =
        await _channel.invokeMethod('setUserId', {"userId": userId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// If users of Appmate use One Signal services to receive push notifications,
  /// they must be able to set their One Signal ids via setOneSignalId.
  /// So, they will be able to receive push notifications relating to appmate.
  ///
  /// @param _oneSignalId [String] Device OneSignal Id information.
  static Future<BaseResponse> setOneSignalId(String oneSignalId) async {
    List<Object?>? response = await _channel
        .invokeMethod('setOneSignalId', {"oneSignalId": oneSignalId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### Returns device user id.
  ///
  /// User id can retrieve from Appmate SDK regardless of
  /// you set a userId or a random userId is created.
  ///
  /// Callbacks user id as [String]
  static Future<String> getUserId() async {
    final String? result = await _channel.invokeMethod('getUserId');
    if (result != null) {
      return result;
    }
    return "";
  }

  /// Huawei devices supports Huawei Mobile Services (HMS) and
  /// other Android devices supports Google Mobile Services (GMS).
  /// Some Huawei devices supports both of HMS and GMS.
  /// In this situation appmate SDK works over GMS.
  /// But developers can set running platform type by calling setPlatformType method.
  /// If the device is not supporting the passed platform type, onError will be triggered.
  ///
  /// [platformType] Platform type
  ///
  /// Callbacks [BaseResponse] and [GenericError] in case of failure
  static Future<BaseResponse> setPlatformType(String platformType) async {
    if (!Platform.isAndroid) {
      BaseResponse result =
          new BaseResponse(false, GenericError('Unsupported platform!'));
      return result;
    }
    List<Object?>? response = await _channel
        .invokeMethod('setPlatformType', {"platformType": platformType});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// By default appmate SDK return platform type depending on which environment is supported
  ///
  /// The platform type will be returned as the [result]
  static Future<String> getPlatformType() async {
    if (!Platform.isAndroid) {
      return "";
    }
    String response = await _channel.invokeMethod('getPlatformType');
    return response;
  }

  static Future<BaseResponse> setRuntimeEnvironment(String environment) async {
    List<Object?>? response = await _channel
        .invokeMethod('setRuntimeEnvironment', {"environment": environment});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### Fetches all the products created on appmate server. If sandbox parameter value is false, non-synched products will not be fetched.
  ///
  /// If a previous successful fetch exists in last 10 minutes,
  /// it fetches the products from local cache.
  ///
  /// So you will be able to get the response faster than getting directly from the network.
  /// Otherwise, it will launch a network request.
  ///
  /// Callbacks [ProductsResponse] and [GenericError] in case of failure.
  static Future<ProductsResponse> getProducts() async {
    final List<Object?>? response = await _channel.invokeMethod('getProducts');
    return _getProductsFromResponse(response);
  }

  /// #### Fetches the filtered products with specified type from appmate server. If sandbox parameter value is false, non-synched products will not be fetched
  ///
  /// If a previous successful fetch exists in last 10 minutes, it fetches the products from local cache.
  /// So you will be able to get the response faster than getting directly from the network. Otherwise, it will launch a network request.
  ///
  /// If you add a new product and want to see it in your app for testing purposes, you can delete your app's cache.
  /// The products will be fetched from the network the next time you open your app.
  ///
  /// [type] of products as String. It must be typed as "0" for consumables, "1" for non-consumables and "2" for subscriptions.
  ///
  /// Callbacks [ProductsResponse] and [GenericError] in case of failure.
  static Future<ProductsResponse> getProductsWithType(String type) async {
    final List<Object?>? response =
        await _channel.invokeMethod('getProductsWithType', {"type": type});
    return _getProductsFromResponse(response);
  }

  /// #### Fetches the filtered products with specified id list from appmate server. If sandbox parameter value is false, non-synched products will not be fetched
  ///
  /// If a previous successful fetch exists in last 10 minutes, it fetches the products from local cache.
  /// So you will be able to get the response faster than getting directly from the network. Otherwise, it will launch a network request.
  ///
  /// If you add a new product and want to see it in your app for testing purposes, you can delete your app's cache.
  /// The products will be fetched from the network the next time you open your app.
  ///
  /// List of desired product ids as String array[ids].
  ///
  /// Callbacks [ProductsResponse] and [GenericError] in case of failure.
  static Future<ProductsResponse> getProductsWithIdList(
      List<String> ids) async {
    final List<Object?>? response =
        await _channel.invokeMethod('getProductsWithIdList', {"ids": ids});
    return _getProductsFromResponse(response);
  }

  static ProductsResponse _getProductsFromResponse(List<Object?>? response) {
    if (response != null) {
      List<Product>? products;
      GenericError? error;
      if (response[0] != null) {
        products = List<Product>.from(json
            .decode(response[0]!.toString())
            .map((model) => Product.fromJson(model)));
      }
      if (response[1] != null) {
        error = GenericError.fromJson(json.decode(response[1]!.toString()));
      }
      return new ProductsResponse(products, error);
    }
    return new ProductsResponse(null, null);
  }

  /// #### Fetches all the purchases made so far from appmate server.
  ///
  /// If a previous successful fetch exists in last 10 minutes, it fetches the products from local cache.
  /// So you will be able to get the response faster than getting directly from the network. Otherwise, it will launch a network request.
  ///
  /// By calling getCurrentPurchases method, you can get the list of purchases filtered by specific rules. These filtering rules are;
  /// - Purchased but not consumed consumable products.
  /// - Non-consumable products.
  /// - Ongoing subscription products.
  ///
  /// Purchases which are made before/without appmate will still be returned if they match these rules.
  ///
  /// Callbacks [PurchasesResponse] and [GenericError] in case of failure.
  static Future<PurchasesResponse> getCurrentPurchases() async {
    List<Object?>? response =
        await _channel.invokeMethod('getCurrentPurchases');
    return _getPurchasesFromResponse(response);
  }

  /// #### Fetches all the purchases made so far from appmate server.
  /// If a previous successful fetch exists in last 10 minutes, it fetches the products from local cache.
  /// So you will be able to get the response faster than getting directly from the network. Otherwise, it will launch a network request.
  ///
  /// By calling getAllPurchases method, you can get the list of purchases made so far.
  ///
  /// Purchases which are made before/without appmate will still be returned.
  ///
  /// Callbacks [PurchasesResponse] and [GenericError] in case of failure.
  static Future<PurchasesResponse> getAllPurchases() async {
    List<Object?>? response = await _channel.invokeMethod('getAllPurchases');
    return _getPurchasesFromResponse(response);
  }

  /// #### Restores all the previous purchases of current user.
  ///
  /// If user has purchases from previous installations of the application, it is possible to restore them by using this method.
  ///
  /// Apple obliges developers to add a restore button in their applications.
  ///
  /// Android doesn't require a restore function. So, the method will not work on Android devices.
  ///
  /// Callbacks [PurchasesResponse] and [GenericError] in case of failure.
  static Future<PurchasesResponse> restorePurchases() async {
    List<Object?>? response = await _channel.invokeMethod('restorePurchases');
    return _getPurchasesFromResponse(response);
  }

  static PurchasesResponse _getPurchasesFromResponse(List<Object?>? response) {
    if (response != null) {
      List<PurchaseInfo>? purchases;
      GenericError? error;
      if (response[0] != null) {
        purchases = List<PurchaseInfo>.from(json
            .decode(response[0]!.toString())
            .map((model) => PurchaseInfo.fromJson(model)));
      }
      if (response[1] != null) {
        error = GenericError.fromJson(json.decode(response[1]!.toString()));
      }
      return new PurchasesResponse(purchases, error);
    }
    return new PurchasesResponse(null, null);
  }

  /// #### Launches a new purchase transaction using Apple interface.
  ///
  /// When you want to make a purchase transaction, just pass the Product object parameter to the method,
  /// then appmate will communicate with AppStore and handle Apple purchase process.
  ///
  /// Provide the [Product] object you want to purchase.
  ///
  /// Callbacks [MakePurchaseResponse] and [GenericError] in case of failure.
  static Future<MakePurchaseResponse> makePurchase(Product product) async {
    List<Object?>? response = await _channel.invokeMethod(
        'makePurchase', {"product": json.encode(product.toJson())});
    MakePurchaseResponse? result = new MakePurchaseResponse(null, null);
    if (response != null) {
      if (response[0] != null) {
        var jsonResponse = json.decode(response[0]!.toString());
        if (jsonResponse["purchaseInfo"] is List) {
          result =
              MakePurchaseResponse.fromJson(jsonResponse["purchaseInfo"][0]);
        } else {
          result = MakePurchaseResponse.fromJson(jsonResponse);
        }
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new MakePurchaseResponse(null, error);
      }
      return result;
    }
    return result;
  }

  /// #### Consume the purchased product specified with purchaseToken.
  ///
  /// Once purchase is completed, as a part of purchase process you need to deliver a content with granting entitlement as a result of the user's purchase.
  /// For one-time consumable product, you need to call consume method of the SDK to indicate that your app has granted entitlement to the user.
  /// With consumption of the one-time consumable product, the product can be purchased again.
  ///
  /// [purchaseToken] Token created after a purchase. It can be found inside PurchaseInfo object.
  ///
  /// Callbacks [boolean] and [GenericError] in case of failure.
  static Future<BaseResponse> consumePurchase(String purchaseToken) async {
    List<Object?>? response = await _channel
        .invokeMethod('consumePurchase', {"purchaseToken": purchaseToken});
    BaseResponse? result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### Consume a list of purchased product specified with purchaseToken list.
  ///
  /// Once purchase is completed, as a part of purchase process you need to deliver a content with granting entitlement as a result of the user's purchase.
  /// For one-time consumable product, you need to call consume method of the SDK to indicate that your app has granted entitlement to the user.
  /// With consumption of the one-time consumable product, the product can be purchased again.
  ///
  /// After consumption, the product will vanish in current purchases but will still exist in purchase history.
  ///
  /// [purchaseTokens] Tokens as String array created after a purchase. It can be found inside PurchaseInfo object.
  ///
  /// Callbacks [BaseResponse] and [GenericError] in case of failure
  static Future<BaseResponse> consumePurchases(
      List<String> purchaseTokens) async {
    List<Object?>? response = await _channel
        .invokeMethod('consumePurchases', {"purchaseTokens": purchaseTokens});
    BaseResponse? result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### Creates a relation between current user and another user.
  ///
  /// This method allows developers to establish a connection between users. Thanks to this relation, sub-user will also own all the purchases  master user have made.
  ///
  /// For example, you have a user who has two devices and he/she wants to use your app with both of them.
  /// If your user has some purchases that made with the first userId, also second userId will be able to see this products as purchased.
  /// Whenever there is a change in a user's entitlement to products within your app, the second device user also will be able to get this updates.
  /// It's easy to use this feature with appmate only by establishing relationships between the users.
  ///
  /// [masterUserId] Id of the user has the purchases of his/her own. User id can be retrieved via *getUserId* method.
  ///
  /// Callbacks [CreateUserRelationResponse] and [GenericError] in case of failure.
  static Future<CreateUserRelationResponse> createUserIdRelation(
      String masterUserId) async {
    List<Object?>? response = await _channel
        .invokeMethod('createUserIdRelation', {"masterUserId": masterUserId});
    CreateUserRelationResponse result =
        new CreateUserRelationResponse(null, null);
    if (response != null) {
      if (response[0] != null) {
        UserRelation userRelation =
            UserRelation.fromJson(json.decode(response[0].toString()));
        result = new CreateUserRelationResponse(userRelation, null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new CreateUserRelationResponse(null, error);
      }
      return result;
    }
    return result;
  }

  /// #### Removes all the existing relations of a master.
  ///
  /// If you want to delete all the relationships related to the masterUserId, you can use deleteMasterUserIdRelation method by giving masterUserId as a parameter.
  ///
  /// [masterUserId] Id of the user has the purchases of his/her own. User id can be retrieved via *getUserId* method.
  ///
  /// Callbacks [boolean] and [GenericError] in case of failure.
  static Future<BaseResponse> deleteMasterUserIdRelation(
      String masterUserId) async {
    List<Object?>? response = await _channel.invokeMethod(
        'deleteMasterUserIdRelation', {"masterUserId": masterUserId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### Removes the existing relation of a sub-user.
  ///
  /// It's also possible to delete a relationship between the users.
  /// By giving subUserId as a parameter, you can use deleteSubUserIdRelation method and delete a relationship between subUserId and its master.
  ///
  /// [subUserId] Id of the user has the purchases of another user. User id can be retrieved via *getUserId* method.
  ///
  /// Callbacks [boolean] and [GenericError] in case of failure.
  static Future<BaseResponse> deleteSubUserIdRelation(String subUserId) async {
    List<Object?>? response = await _channel
        .invokeMethod('deleteSubUserIdRelation', {"subUserId": subUserId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### For iOS platform:
  /// Navigates to Subscriptions page of the device.
  /// Apple does not allow developer to end subscriptions directly; instead, directs to the Subscriptions page of the device.

  /// #### For Android platform:
  /// Ends an ongoing subscription.
  ///
  /// [purchaseToken] Token created after a purchase. It can be found inside PurchaseInfo object.
  ///
  /// Callbacks [boolean] and [GenericError] in case of failure.
  static Future<BaseResponse> unsubscribe(String purchaseToken) async {
    List<Object?>? response = await _channel
        .invokeMethod('unsubscribe', {"purchaseToken": purchaseToken});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// #### Checks whether a product is actively purchased or not.
  ///
  /// With calling [isProductPurchased] method of the SDK you can check whether a product is purchased or not.
  /// Once you query a product with productId, a boolean response comes back.
  ///
  /// True means that the product is purchased. After consumption, it will respond a false value.
  /// These are accepted as purchased:
  ///    - Purchased but not consumed consumable products.
  ///    - Non-consumable products.
  ///    - Ongoing subscription products.
  ///
  /// [productId] Id of the product to be checked.
  ///
  /// Callbacks [boolean] and [GenericError] in case of failure.
  static Future<BaseResponse> isProductPurchased(String productId) async {
    List<Object?>? response = await _channel
        .invokeMethod('isProductPurchased', {"productId": productId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  /// getEntitlements method
  static Future<EntitlementResponse> getEntitlements() async {
    List<Object?>? response = await _channel.invokeMethod('getEntitlements');
    if (response != null) {
      List<String>? entitlements;
      GenericError? error;
      if (response[0] != null) {
        entitlements = (response[0] as List).cast<String>();
      }
      if (response[1] != null) {
        error = GenericError.fromJson(json.decode(response[1]!.toString()));
      }
      return new EntitlementResponse(entitlements, error);
    }
    return new EntitlementResponse(null, null);
  }

  /// setAppmateEvent method
  static Future<BaseResponse> setAppmateEvent(
      String productId, UserEventType userEventType) async {
    String? userEvent = null;
    if (userEventType == UserEventType.FIRST_LAUNCH) {
      userEvent = '1';
    } else if (userEventType == UserEventType.VIEW) {
      userEvent = '2';
    } else if (userEventType == UserEventType.PURCHASE) {
      userEvent = '3';
    }

    List<Object?>? response = await _channel.invokeMethod(
        'setAppmateEvent', {"productId": productId, 'userEvent': userEvent});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  ///  set the Identifier for Advertisers assigned by Apple to a user’s device.
  ///  If users of Appmate has integration of the AppsFlyer, they can set the AppsFlyerId value via this method.
  ///  - Parameters:
  ///     - ADID: the Identifier for Advertisers.
  static Future<BaseResponse> setAdvertiseId(String ADID) async {
    List<Object?>? response =
        await _channel.invokeMethod('setAdvertiseId', {"ADID": ADID});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  ///  Modifies  AppsFlyer identification.
  ///  If users of Appmate has integration of the AppsFlyer, they can set the AppsFlyerId value via this method
  ///  - Parameters:
  ///     - appsFlyerId: the identification of the  AppsFlyer app configuration

  static Future<BaseResponse> setAppsFlyerId(String appsFlyerId) async {
    List<Object?>? response = await _channel
        .invokeMethod('setAppsFlyerId', {"appsFlyerId": appsFlyerId});
    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  ///    allows user to add a set of  dynamic User Specific Attributes.
  ///    - Parameters:
  ///       - userAttribute: any set of information that the user needs to add for example (userName, userEmail etc ...)
  static Future<BaseResponse> setUserAttributes(
      Map<String, Object> userAttributes) async {
    List<Object?>? response = await _channel
        .invokeMethod('setUserAttributes', {"userAttributes": userAttributes});

    BaseResponse result = new BaseResponse(false, null);
    if (response != null) {
      if (response[0] != null) {
        result = new BaseResponse(response[0].toString() == "true", null);
      }
      if (response[1] != null) {
        GenericError error =
            GenericError.fromJson(json.decode(response[1]!.toString()));
        result = new BaseResponse(false, error);
      }
      return result;
    }
    return result;
  }

  static Future<OfferwallResponse> getOfferwall(String? offeringId) async {
    List<Object?>? response =
        await _channel.invokeMethod('getOfferwall', {"offeringId": offeringId});
    if (response != null) {
      Offerwall? offerwall;
      GenericError? error;
      if (response [0] != null && response[0] != false) {
        offerwall = Offerwall.fromJson(json.decode(response[0]!.toString()));
      }
      if (response[1] != null) {
        error = GenericError.fromJson(json.decode(response[1]!.toString()));
      }
      return new OfferwallResponse(offerwall, error);
    }
    return new OfferwallResponse(null, null);
  }

  ///     Use this method to enable/disable the debug logs of the Appmate framework. The default parameter is false.
  ///    - Parameters:
  ///        - enable: Set to true to show Appmate logs.
  static void enableDebugLogs(bool enable) async {
    // todo: create a Logger class to log all the incoming data from the native bridgs and make this as a flag to log or ignore the case.
    _channel.invokeMethod('enableDebugLogs', {"enable": enable});
  }
}
